--[[
Name: LibSQL-1.0
Revision: $Rev: 4 $
Author(s): Humbedooh
Description: SQL library for Lua
Optional dependencies: LibStub
License: All Sporks Reserved!
]]
-----------------------------------------------------------------------

local LIBSQL = "LibSQL-1.0"
local LIBSQL_MINOR = tonumber(("$Rev: 04 $"):match("(%d+)")) or 10000;
local LibSQL = false;
if LibStub then
    LibSQL = LibStub:NewLibrary(LIBSQL, LIBSQL_MINOR)
    _G.LibSQL = LibSQL;
else
    if not _G.LibSQL or (type(_G.LibSQL) == "table" and _G.LibSQL.rev < LIBSQL_MINOR) then
        _G.LibSQL = {rev = LIBSQL_MINOR};
    end
    LibSQL = _G.LibSQL;
end
if not LibSQL then return end
LibSQL_Functions = {'ABS', 'ACOS', 'ADDDATE', 'ASCII', 'ASIN', 'ATAN', 'ATAN2', 'AVG', 'BIN', 'BIT_AND', 'BIT_COUNT', 'BIT_LENGTH', 'BIT_OR', 'CAST', 'CEIL', 'CEILING', 'CHAR_LENGTH', 'CHARACTER_LENGTH', 'CONCAT', 'CONCAT_WS', 'CONV', 'CONVERT', 'COS', 'COT', 'COUNT', 'CURDATE', 'CURRENT_DATE', 'CURRENT_TIME', 'CURRENT_TIMESTAMP', 'CURTIME', 'DATE_ADD', 'DATE_FORMAT', 'DATE_SUB', 'DAYNAME', 'DAYOFMONTH', 'DAYOFWEEK', 'DAYOFYEAR', 'DECODE', 'DEGREES', 'ENCODE', 'ENCRYPT', 'EXP', 'EXPORT_SET', 'EXTRACT', 'FIELD', 'FIND_IN_SET', 'FLOOR', 'FORMAT', 'FOUND_ROWS', 'FROM_DAYS', 'FROM_UNIXTIME', 'GREATEST', 'HEX', 'IFNULL', 'INTERVAL', 'ISNULL', 'LAST_INSERT_ID', 'LCASE', 'LEAST', 'LEFT', 'LENGTH', 'LN', 'LOCATE', 'LOG', 'LOG2', 'LOG10', 'LOWER', 'LPAD', 'LTRIM', 'MAKE_SET', 'MAX', 'MD5', 'MID', 'MIN', 'MOD', 'MONTHNAME', 'NOW', 'NULLIF', 'OCT', 'OCTET_LENGTH', 'ORD', 'PASSWORD', 'PERIOD_ADD', 'PERIOD_DIFF', 'PI', 'POSITION', 'POW', 'POWER', 'QUARTER', 'QUOTE', 'RADIANS', 'RAND', 'RELEASE_LOCK', 'REPEAT', 'REVERSE', 'RIGHT', 'ROUND', 'RPAD', 'RTRIM', 'SEC_TO_TIME', 'SHA', 'SHA1', 'SIGN', 'SIN', 'SPACE', 'SQRT', 'STD', 'STDDEV', 'STRCMP', 'SUBDATE', 'SUBSTRING', 'SUBSTRING_INDEX', 'SUM', 'SYSDATE', 'TAN', 'TIME_FORMAT', 'TIME_TO_SEC', 'TO_DAYS', 'TRIM', 'UCASE', 'UNIX_TIMESTAMP', 'UPPER', 'WEEK', 'WEEKDAY', 'YEARWEEK'};
LibSQL_Field_Flags = {'AUTO_INCREMENT','BINARY','DEFAULT','NATIONAL','PRECISION','UNSIGNED','VARYING','ZEROFILL'};
LibSQL_Reserved = {'ACTION','ADD','AFTER','AGAINST','AGGREGATE','ALL','ALTER','ANALYSE','ANALYZE','AND','AS','ASC','AUTOCOMMIT','AVG_ROW_LENGTH','BACKUP','BEGIN','BETWEEN','BOTH','BY','CASCADE','CASE','CHANGE','CHANGED','CHECK','CHECKSUM','CLIENT','COLUMN','COLUMNS','COMMENT','COMMIT','COMMITTED','COMPRESSED','CONCURRENT','CONSTRAINT','CREATE','CROSS','DATA','DATABASE','DATABASES','DAY','DAY_HOUR','DAY_MINUTE','DAY_SECOND','DELAYED','DELAY_KEY_WRITE','DELETE','DESC','DESCRIBE','DISTINCT','DISTINCTROW','DO','DROP','DUMPFILE','DYNAMIC','ELSE','ENCLOSED','END','ESCAPE','ESCAPED','EXECUTE','EXISTS','EXPLAIN','EXTENDED','FAST','FIELDS','FILE','FIRST','FIXED','FLUSH','FOR','FOREIGN','FROM','FULL','FULLTEXT','FUNCTION','GEMINI','GEMINI_SPIN_RETRIES','GLOBAL','GRANT','GRANTS','GROUP','HAVING','HEAP','HIGH_PRIORITY','HOSTS','HOUR','HOUR_MINUTE','HOUR_SECOND','IDENTIFIED','IF','IGNORE','IN','INDEX','INDEXES','INFILE','INNER','INSERT','INSERT_ID','INSERT_METHOD','INTERVAL','INTO','IS','ISOLATION','JOIN','KEY','KEYS','KILL','LAST_INSERT_ID','LEADING','LEFT','LEVEL','LIKE','LIMIT','LINES','LOAD','LOCAL','LOCK','LOCKS','LOGS','LOW_PRIORITY','MASTER','MASTER_CONNECT_RETRY','MASTER_HOST','MASTER_LOG_FILE','MASTER_LOG_POS','MASTER_PASSWORD','MASTER_PORT','MASTER_USER','MATCH','MAX_CONNECTIONS_PER_HOUR','MAX_QUERIES_PER_HOUR','MAX_ROWS','MAX_UPDATES_PER_HOUR','MEDIUM','MERGE','MIN_ROWS','MINUTE','MINUTE_SECOND','MODE','MODIFY','MONTH','MRG_MYISAM','MYISAM','NATURAL','NO','NOT','NULL','ON','OPEN','OPTIMIZE','OPTION','OPTIONALLY','OR','ORDER','OUTER','OUTFILE','PACK_KEYS','PARTIAL','PASSWORD','PRIMARY','PRIVILEGES','PROCEDURE','PROCESS','PROCESSLIST','PURGE','QUICK','RAID0','RAID_CHUNKS','RAID_CHUNKSIZE','RAID_TYPE','READ','REFERENCES','REGEXP','RELOAD','RENAME','REPAIR','REPEATABLE','REPLACE','REPLICATION','RESET','RESTORE','RESTRICT','RETURNS','REVOKE','RIGHT','RLIKE','ROLLBACK','ROW','ROW_FORMAT','ROWS','SECOND','SELECT','SERIALIZABLE','SESSION','SHARE','SHOW','SHUTDOWN','SLAVE','SONAME','SQL_AUTO_IS_NULL','SQL_BIG_RESULT','SQL_BIG_SELECTS','SQL_BIG_TABLES','SQL_BUFFER_RESULT','SQL_LOG_BIN','SQL_LOG_OFF','SQL_LOG_UPDATE','SQL_LOW_PRIORITY_UPDATES','SQL_MAX_JOIN_SIZE','SQL_QUOTE_SHOW_CREATE','SQL_SAFE_UPDATES','SQL_SELECT_LIMIT','SQL_SLAVE_SKIP_COUNTER','SQL_SMALL_RESULT','SQL_WARNINGS','START','STARTING','STATUS','STOP','STRAIGHT_JOIN','STRING','STRIPED','SUPER','TABLE','TABLES','TEMPORARY','TERMINATED','THEN','TO','TRAILING','TRANSACTION','TRUNCATE','TYPE','UNCOMMITTED','UNION','UNIQUE','UNLOCK','UPDATE','USAGE','USE','USING','VALUES','VARIABLES','WHEN','WHERE','WITH','WORK','WRITE','YEAR_MONTH'};
LibSQL_Field_Types = {'INTEGER', 'NUMBER', 'STRING', 'BOOLEAN', 'ENUM', 'DATE', 'TIMESTAMP', 'BLOB'}

LibSQL_Resource = {};
function LibSQL:Parse(arg)
    local db = self;
    if type(arg) ~= "string" or type(db) ~= "table" then return end
    local cmd = arg:match("^%s*(%a+)%s");
    if cmd then
        cmd = cmd:upper();
    end
end

local function LibSQL_Resource_Next(res)
    res._pointer = (res._pointer or 0)+1;
    local entry = res._entries[res._pointer];
    if not entry then return nil end
    return unpack(entry or {});
end

local function LibSQL_Resource_Next_Arrayed(res)
    res._pointer = (res._pointer or 0)+1;
    local entry = res._entries[res._pointer];
    if not entry then return nil end
    return entry or {};
end

local function LibSQL_Resource_Next_Objectified(res)
    res._pointer = (res._pointer or 0)+1;
    local entry = res._entries[res._pointer];
    local obj = {};
    if not entry then return nil end
    local f = res._fields;
    for n = 1, #f do
        obj[f[n]] = entry[n];
    end
    return obj;
end


function LibSQL_Resource:Rewind() self._pointer = nil end
function LibSQL_Resource:Seek(num) self._pointer = tonumber(num) return self._pointer end
function LibSQL_Resource:Rows() return #self._entries end
function LibSQL_Resource:Fetch() return LibSQL_Resource_Next, self, nil end
function LibSQL_Resource:FetchObject() return LibSQL_Resource_Next_Objectified, self, nil end
function LibSQL_Resource:FetchArray() return LibSQL_Resource_Next_Arrayed, self, nil end

function LibSQL_Resource:New()
    local res = {};
    res._fields =   {};
    res._entries =  {}
    res._pointer = nil;
    res.Fetch =             LibSQL_Resource.Fetch;
    res.FetchObject =       LibSQL_Resource.FetchObject;
    res.FetchArray =        LibSQL_Resource.FetchArray;
    res.Rewind =            LibSQL_Resource.Rewind;
    res.Seek =              LibSQL_Resource.Seek;
    res.Rows =              LibSQL_Resource.Rows;
    return res;
end

function LibSQL:Open(db)
    db.Execute = LibSQL.Execute;
    return db;
end

function LibSQL:New() return LibSQL:Open({_tables = {}}) end

--db:select("myTable"):find("name=Moo"):sort("ASC"):limit(0,10):select("name", "level");
--[[

local db = LibSQL_Resource:New();
print("got", db:Rows(), "results!");
for a,b,c,d in db:FetchObject() do
    print(a.name, a.level)
end

]]--
